@@ -19,6 +19,9 @@ pub unsafe trait DynamicBundle {
         None
     }
 
+    /// Returns true if bundle has speicifed type id.
+    fn contains_id(&self, id: TypeId) -> bool;
+
     /// Calls provided closure with slice of ids of types that this bundle contains.
     fn with_ids<R>(&self, f: impl FnOnce(&[TypeId]) -> R) -> R;
 
@@ -38,7 +41,10 @@ pub trait Bundle: DynamicBundle {
     /// Returns static key for the bundle type.
     fn static_key() -> TypeId;
 
-    /// Calls provided closure with slice of ids of types that this bundle contains.    
+    /// Returns true if bundle has speicifed type id.
+    fn static_contains_id(id: TypeId) -> bool;
+
+    /// Calls provided closure with slice of ids of types that this bundle contains.
     fn static_with_ids<R>(f: impl FnOnce(&[TypeId]) -> R) -> R;
 
     /// Calls provided closure with slice of component infos of types that this bundle contains.
@@ -69,6 +75,11 @@ macro_rules! for_tuple {
                 Some(Self::static_key())
             }
 
+            #[inline]
+            fn contains_id(&self, id: TypeId) -> bool {
+                Self::static_contains_id(id)
+            }
+
             #[inline]
             fn with_ids<R>(&self, f: impl FnOnce(&[TypeId]) -> R) -> R {
                 Self::static_with_ids(f)
@@ -90,6 +101,11 @@ macro_rules! for_tuple {
                 TypeId::of::<()>()
             }
 
+            #[inline]
+            fn static_contains_id(_id: TypeId) -> bool {
+                false
+            }
+
             #[inline]
             fn static_with_ids<R>(f: impl FnOnce(&[TypeId]) -> R) -> R {
                 f(&[])
@@ -116,6 +132,11 @@ macro_rules! for_tuple {
                 Some(<Self as Bundle>::static_key())
             }
 
+            #[inline]
+            fn contains_id(&self, id: TypeId) -> bool {
+                <Self as Bundle>::static_contains_id(id)
+            }
+
             #[inline]
             fn with_ids<R>(&self, f: impl FnOnce(&[TypeId]) -> R) -> R {
                 <Self as Bundle>::static_with_ids(f)
@@ -162,6 +183,11 @@ macro_rules! for_tuple {
                 TypeId::of::<Self>()
             }
 
+            #[inline]
+            fn static_contains_id(id: TypeId) -> bool {
+                $( TypeId::of::<$a>() == id )|| *
+            }
+
             #[inline]
             fn static_with_ids<R>(f: impl FnOnce(&[TypeId]) -> R) -> R {
                 f(&[$(TypeId::of::<$a>(),)+])
