@@ -3,10 +3,16 @@
 pub use crate::{
     bundle::{Bundle, DynamicBundle},
     component::Component,
-    entity::{Entity, EntityId, WeakEntity},
+    entity::EntityId,
     query::{
         Alt, ImmutableQuery, Modifed, NonTrackingQuery, Query, QueryItem, QueryIter,
         QueryTrackedIter,
     },
     world::{EntityError, Tracks, World},
 };
+
+#[cfg(feature = "rc")]
+pub use crate::{
+    entity::{Entity, SharedEntity},
+    proof::Skip,
+};
