@@ -183,7 +183,10 @@ where
         epoch: u64,
     ) -> Option<ModifiedFetchWrite<T>> {
         let idx = archetype.id_index(TypeId::of::<T>())?;
-        let data = archetype.data_mut(idx);
+        let data = archetype.data(idx);
+
+        debug_assert!(*data.version.get() < epoch);
+        *data.version.get() = epoch;
 
         Some(ModifiedFetchWrite {
             tracks,
@@ -264,11 +267,11 @@ where
     #[inline]
     unsafe fn fetch(archetype: &Archetype, tracks: u64, epoch: u64) -> Option<ModifiedFetchAlt<T>> {
         let idx = archetype.id_index(TypeId::of::<T>())?;
-        let data = archetype.data_mut(idx);
+        let data = archetype.data(idx);
         debug_assert_eq!(data.id, TypeId::of::<T>());
 
-        debug_assert!(data.version < epoch);
-        data.version = epoch;
+        debug_assert!(*data.version.get() < epoch);
+        *data.version.get() = epoch;
 
         Some(ModifiedFetchAlt {
             tracks,
